@@ -305,7 +305,7 @@ int cb_instance_modify_config_check_callback(Slapi_PBlock *pb, Slapi_Entry* entr
         slapi_pblock_get( pb, SLAPI_MODIFY_MODS, &mods );
 
 	/* First pass to validate input */
-        for (i = 0; mods[i] && LDAP_SUCCESS == rc; i++) {
+        for (i = 0; mods && mods[i] && LDAP_SUCCESS == rc; i++) {
                 attr_name = mods[i]->mod_type;
 
 		/* specific processing for multi-valued attributes */
@@ -378,7 +378,7 @@ int cb_instance_modify_config_callback(Slapi_PBlock *pb, Slapi_Entry* entryBefor
 
 	/* input checked in the preop modify callback */
 
-        for (i = 0; mods[i] && LDAP_SUCCESS == rc; i++) {
+        for (i = 0; mods && mods[i] && LDAP_SUCCESS == rc; i++) {
                 attr_name = mods[i]->mod_type;
 
 		/* specific processing for multi-valued attributes */
@@ -405,7 +405,7 @@ int cb_instance_modify_config_callback(Slapi_PBlock *pb, Slapi_Entry* entryBefor
                                	} else
                                	if (SLAPI_IS_MOD_DELETE(mods[i]->mod_op)) {
                                        	charray_remove(inst->illegal_attributes,
-                                               	slapi_ch_strdup(config_attr_value),
+                                               	config_attr_value,
 												0 /* freeit */);
                                	}
                         }
@@ -417,43 +417,40 @@ int cb_instance_modify_config_callback(Slapi_PBlock *pb, Slapi_Entry* entryBefor
                         continue;
 		} 
 		if ( !strcasecmp ( attr_name, CB_CONFIG_CHAINING_COMPONENTS )) {
-               		char * config_attr_value;
-                       	int done=0;
+			char *config_attr_value;
+			char *attr_val;
+			int done=0;
 			int j;
 
-        		slapi_rwlock_wrlock(inst->rwl_config_lock);
-                       	for (j = 0; mods[i]->mod_bvalues && mods[i]->mod_bvalues[j]; j++) {
-                               	config_attr_value = (char *) mods[i]->mod_bvalues[j]->bv_val;
-                               	if (SLAPI_IS_MOD_REPLACE(mods[i]->mod_op)) {
-                                       	if (!done) {
-                                               	charray_free(inst->chaining_components);
-                                               	inst->chaining_components=NULL;
-                                               	done=1;
-                                       	}
+			slapi_rwlock_wrlock(inst->rwl_config_lock);
+			for (j = 0; mods[i]->mod_bvalues && mods[i]->mod_bvalues[j]; j++) {
+				config_attr_value = (char *) mods[i]->mod_bvalues[j]->bv_val;
+				if (SLAPI_IS_MOD_REPLACE(mods[i]->mod_op)) {
+					if (!done) {
+						charray_free(inst->chaining_components);
+						inst->chaining_components=NULL;
+						done=1;
+					}
 					/* XXXSD assume dns */
-                                       	charray_add(&inst->chaining_components,
-                                               	slapi_dn_normalize(slapi_ch_strdup(config_attr_value)));
-                               	} else
-                               	if (SLAPI_IS_MOD_ADD(mods[i]->mod_op)) {
-                                       	charray_add(&inst->chaining_components,
-                                               	slapi_dn_normalize(slapi_ch_strdup(config_attr_value)));
-                               	} else
-                               	if (SLAPI_IS_MOD_DELETE(mods[i]->mod_op)) {
-                                       	charray_remove(inst->chaining_components,
-                                               	slapi_dn_normalize(slapi_ch_strdup(config_attr_value)),
-												0 /* freeit */);
-                               	}
-                       	}
-                       	if (NULL == mods[i]->mod_bvalues) {
-                               	charray_free(inst->chaining_components);
-                               	inst->chaining_components=NULL;
-                       	}
-        		slapi_rwlock_unlock(inst->rwl_config_lock);
-                        continue;
+					charray_add(&inst->chaining_components,
+									slapi_dn_normalize(slapi_ch_strdup(config_attr_value)));
+				} else if (SLAPI_IS_MOD_ADD(mods[i]->mod_op)) {
+					charray_add(&inst->chaining_components,
+								slapi_dn_normalize(slapi_ch_strdup(config_attr_value)));
+				} else if (SLAPI_IS_MOD_DELETE(mods[i]->mod_op)) {
+					attr_val = slapi_dn_normalize(slapi_ch_strdup(config_attr_value));
+					charray_remove(inst->chaining_components,attr_val, 0 /* freeit */);
+					slapi_ch_free_string(&attr_val);
+				}
+			}
+			if (NULL == mods[i]->mod_bvalues) {
+				charray_free(inst->chaining_components);
+				inst->chaining_components=NULL;
+			}
+			slapi_rwlock_unlock(inst->rwl_config_lock);
+			continue;
 		} 
 
-
-
 		if (SLAPI_IS_MOD_DELETE(mods[i]->mod_op) || 
                         SLAPI_IS_MOD_ADD(mods[i]->mod_op)) {
 
@@ -741,11 +738,11 @@ static int cb_instance_hosturl_set(void *arg, void *value, char *errorbuf, int p
 
 	if ((LDAP_SUCCESS == rc) && apply) {
 
-               	slapi_rwlock_wrlock(inst->rwl_config_lock);
-
-	        if (( phase != CB_CONFIG_PHASE_INITIALIZATION ) &&
-       			( phase != CB_CONFIG_PHASE_STARTUP )) {
+		slapi_rwlock_wrlock(inst->rwl_config_lock);
 
+		if (( phase != CB_CONFIG_PHASE_INITIALIZATION ) &&
+			( phase != CB_CONFIG_PHASE_STARTUP ))
+		{
 			/* Dynamic modification				*/
 			/* Don't free char * pointer now		*/
 			/* STore them in a waste basket 		*/
@@ -769,17 +766,17 @@ static int cb_instance_hosturl_set(void *arg, void *value, char *errorbuf, int p
 		/* the url and update the configuration  */
 
 		if ((ludp->lud_host==NULL) || (strlen(ludp->lud_host)==0)) {
-                        inst->pool->hostname=(char *)slapi_ch_strdup((char *)get_localhost_DNS());
+			inst->pool->hostname = get_localhost_DNS();
 		} else {
-               		inst->pool->hostname = slapi_ch_strdup( ludp->lud_host );
+			inst->pool->hostname = slapi_ch_strdup( ludp->lud_host );
 		}
-               	inst->pool->url = slapi_ch_strdup( url);
-               	inst->pool->secure = secure;
+		inst->pool->url = slapi_ch_strdup( url);
+		inst->pool->secure = secure;
 
 		if ((ludp->lud_port==0) && inst->pool->secure)
 			inst->pool->port=CB_LDAP_SECURE_PORT;
 		else
-               		inst->pool->port = ludp->lud_port;
+			inst->pool->port = ludp->lud_port;
 
 		/* Build a charray of <host>:<port> */
 		/* hostname is of the form <host>[:port] <host>[:port] */
