@@ -216,6 +216,7 @@ typedef unsigned short u_int16_t;
 #define DEFAULT_IMPORT_INDEX_BUFFER_SIZE  0
 #define SUBLEN                   3
 #define LDBM_CACHE_RETRY_COUNT 1000 /* Number of times we re-try a cache operation */
+#define RETRY_CACHE_LOCK 2 /* error code to signal a retry of the cache lock */
 #define IDL_FETCH_RETRY_COUNT 5 /* Number of times we re-try idl_fetch if it returns deadlock */
 #define IMPORT_SUBCOUNT_HASHTABLE_SIZE 500 /* Number of buckets in hash used to accumulate subcount for broody parents */
 
@@ -402,7 +403,7 @@ struct cache {
     Slapi_Counter *c_tries;
     struct backcommon *c_lruhead;	/* add entries here */
     struct backcommon *c_lrutail;	/* remove entries here */
-    PRLock *c_mutex;			/* lock for cache operations */
+    PRMonitor *c_mutex; 		/* lock for cache operations */  
     PRLock *c_emutexalloc_mutex;
 };
 
@@ -901,7 +902,8 @@ typedef struct _back_search_result_set
 /* operation for parent_update_on_childchange */
 #define PARENTUPDATE_ADD       0x1
 #define PARENTUPDATE_DEL       0x2
-#define PARENTUPDATE_MASK      (PARENTUPDATE_ADD|PARENTUPDATE_DEL)
+#define PARENTUPDATE_RESURECT  0x4
+#define PARENTUPDATE_MASK      (PARENTUPDATE_ADD|PARENTUPDATE_DEL|PARENTUPDATE_RESURECT)
 
 #define PARENTUPDATE_CREATE_TOMBSTONE  0x10
 #define PARENTUPDATE_DELETE_TOMBSTONE  0x20
