@@ -47,6 +47,7 @@
 #ifdef DEBUG
 #define LDAP_CACHE_DEBUG
 /* #define LDAP_CACHE_DEBUG_LRU * causes slowdown */
+/* #define CACHE_DEBUG * causes slowdown */
 #endif
 
 /* cache can't get any smaller than this (in bytes) */
@@ -543,10 +544,9 @@ int cache_init(struct cache *cache, size_t maxsize, long maxentries, int type)
     cache->c_lruhead = cache->c_lrutail = NULL;
     cache_make_hashes(cache, type);
 
-    if (((cache->c_mutex = PR_NewLock()) == NULL) ||
-       ((cache->c_emutexalloc_mutex = PR_NewLock()) == NULL)) {
-       LDAPDebug(LDAP_DEBUG_ANY, "ldbm: cache_init: PR_NewLock failed\n",
-                0, 0, 0);
+    if (((cache->c_mutex = PR_NewMonitor()) == NULL) ||
+        ((cache->c_emutexalloc_mutex = PR_NewLock()) == NULL)) {
+       LDAPDebug0Args(LDAP_DEBUG_ANY, "ldbm: cache_init: PR_NewMonitor failed\n");
        return 0;
     }
     LDAPDebug(LDAP_DEBUG_TRACE, "<= cache_init\n", 0, 0, 0);
@@ -633,13 +633,13 @@ static void entrycache_clear_int(struct cache *cache)
 
 void cache_clear(struct cache *cache, int type)
 {
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (CACHE_TYPE_ENTRY == type) {
         entrycache_clear_int(cache);
     } else if (CACHE_TYPE_DN == type) {
         dncache_clear_int(cache);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 }
 
 static void erase_cache(struct cache *cache, int type)
@@ -663,7 +663,7 @@ void cache_destroy_please(struct cache *cache, int type)
     slapi_counter_destroy(&cache->c_cursize);
     slapi_counter_destroy(&cache->c_hits);
     slapi_counter_destroy(&cache->c_tries);
-    PR_DestroyLock(cache->c_mutex);
+    PR_DestroyMonitor(cache->c_mutex);
     PR_DestroyLock(cache->c_emutexalloc_mutex);
 }
 
@@ -687,7 +687,7 @@ static void entrycache_set_max_size(struct cache *cache, size_t bytes)
                 "WARNING -- Minimum cache size is %lu -- rounding up\n",
                 MINCACHESIZE, 0, 0);
     }
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     cache->c_maxsize = bytes;
     LOG("entry cache size set to %lu\n", bytes, 0, 0);
     /* check for full cache, and clear out if necessary */
@@ -706,7 +706,7 @@ static void entrycache_set_max_size(struct cache *cache, size_t bytes)
        erase_cache(cache, CACHE_TYPE_ENTRY);
        cache_make_hashes(cache, CACHE_TYPE_ENTRY);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     if (! dblayer_is_cachesize_sane(&bytes)) {
        LDAPDebug(LDAP_DEBUG_ANY,
                 "WARNING -- Possible CONFIGURATION ERROR -- cachesize "
@@ -724,7 +724,7 @@ void cache_set_max_entries(struct cache *cache, long entries)
      * was given in # entries instead of memory footprint.  hopefully,
      * we can eventually drop this.
      */
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     cache->c_maxentries = entries;
     if (entries >= 0) {
         LOG("entry cache entry-limit set to %lu\n", entries, 0, 0);
@@ -735,7 +735,7 @@ void cache_set_max_entries(struct cache *cache, long entries)
     /* check for full cache, and clear out if necessary */
     if (CACHE_FULL(cache))
         eflush = entrycache_flush(cache);
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     while (eflush)
     {
         eflushtemp = BACK_LRU_NEXT(eflush, struct backentry *);
@@ -748,9 +748,9 @@ size_t cache_get_max_size(struct cache *cache)
 {
     size_t n = 0;
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     n = cache->c_maxsize;
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     return n;
 }
 
@@ -758,9 +758,9 @@ long cache_get_max_entries(struct cache *cache)
 {
     long n;
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     n = cache->c_maxentries;
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     return n;
 }
 
@@ -786,14 +786,14 @@ void cache_get_stats(struct cache *cache, PRUint64 *hits, PRUint64 *tries,
                      long *nentries, long *maxentries,
                      size_t *size, size_t *maxsize)
 {
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (hits) *hits = slapi_counter_get_value(cache->c_hits);
     if (tries) *tries = slapi_counter_get_value(cache->c_tries);
     if (nentries) *nentries = cache->c_curentries;
     if (maxentries) *maxentries = cache->c_maxentries;
     if (size) *size = slapi_counter_get_value(cache->c_cursize);
     if (maxsize) *maxsize = cache->c_maxsize;
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 }
 
 void cache_debug_hash(struct cache *cache, char **out)
@@ -804,7 +804,7 @@ void cache_debug_hash(struct cache *cache, char **out)
     Hashtable *ht = NULL;
     char *name = "unknown";
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     *out = (char *)slapi_ch_malloc(1024);
     **out = 0;
 
@@ -840,7 +840,7 @@ void cache_debug_hash(struct cache *cache, char **out)
             sprintf(*out + strlen(*out), "%d[%d] ", j, slot_stats[j]);
         slapi_ch_free((void **)&slot_stats);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 }
 
 
@@ -890,7 +890,7 @@ entrycache_remove_int(struct cache *cache, struct backentry *e)
         }
         else
         {
-            LOG("remove %d from id hash failed\n", e->ep_id, 0, 0);
+            LOG("remove %s (%d) from id hash failed\n", ndn, e->ep_id, 0);
         }
     }
 #ifdef UUIDCACHE_ON 
@@ -946,14 +946,14 @@ int cache_remove(struct cache *cache,  void *ptr)
     }
     e = (struct backcommon *)ptr;
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (CACHE_TYPE_ENTRY == e->ep_type) {
         ASSERT(e->ep_refcnt > 0);
         ret = entrycache_remove_int(cache, (struct backentry *)e);
     } else if (CACHE_TYPE_DN == e->ep_type) {
         ret = dncache_remove_int(cache, (struct backdn *)e);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     return ret;
 }
 
@@ -984,7 +984,12 @@ int cache_replace(struct cache *cache, void *oldptr, void *newptr)
 static int entrycache_replace(struct cache *cache, struct backentry *olde,
                               struct backentry *newe)
 {
-    int found;
+    int found = 0;
+    int found_in_dn = 0;
+    int found_in_id = 0;
+#ifdef UUIDCACHE_ON
+    int found_in_uuid = 0;
+#endif
     const char *oldndn;
     const char *newndn;
 #ifdef UUIDCACHE_ON 
@@ -992,6 +997,7 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
     const char *newuuid;
 #endif
     size_t entry_size = 0;
+    struct backentry *alte = NULL;
 
     LOG("=> entrycache_replace (%s) -> (%s)\n", backentry_get_ndn(olde),
         backentry_get_ndn(newe), 0);
@@ -1007,7 +1013,7 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
 #endif
     newndn = slapi_sdn_get_ndn(backentry_get_sdn(newe));
     entry_size = cache_entry_size(newe);
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
 
     /*
      * First, remove the old entry from all the hashtables.
@@ -1015,69 +1021,84 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
      * cache tables, operation error 
      */
     if ( (olde->ep_state & ENTRY_STATE_NOTINCACHE) == 0 ) {
-        int found_in_dn = remove_hash(cache->c_dntable, (void *)oldndn, strlen(oldndn));
-        int found_in_id = remove_hash(cache->c_idtable, &(olde->ep_id), sizeof(ID));
+        found_in_dn = remove_hash(cache->c_dntable, (void *)oldndn, strlen(oldndn));
+        found_in_id = remove_hash(cache->c_idtable, &(olde->ep_id), sizeof(ID));
 #ifdef UUIDCACHE_ON
-        int found_in_uuid = remove_hash(cache->c_uuidtable, (void *)olduuid, strlen(olduuid));
+        found_in_uuid = remove_hash(cache->c_uuidtable, (void *)olduuid, strlen(olduuid));
 #endif
         found = found_in_dn && found_in_id;
 #ifdef UUIDCACHE_ON
         found = found && found_in_uuid;
 #endif
-        if (!found) {
-#ifdef UUIDCACHE_ON
-            LOG("entry cache replace: cache index tables out of sync - found dn [%d] id [%d] uuid [%d]\n",
-                found_in_dn, found_in_id, found_in_uuid);
-#else
-            LOG("entry cache replace: cache index tables out of sync - found dn [%d] id [%d]\n",
-                found_in_dn, found_in_id, 0);
-#endif
-            PR_Unlock(cache->c_mutex);
-            return 1;
-        }
     }
-    if (! entry_same_dn(newe, (void *)oldndn) &&
-         (newe->ep_state & ENTRY_STATE_NOTINCACHE) == 0) {
-        /* if we're doing a modrdn, the new entry can be in the dn table
-         * already, so we need to remove that too.
+    /* If fails, we have to make sure the both entires are removed from the cache,
+     * otherwise, we have no idea what's left in the cache or not... */
+    if (cache_is_in_cache(cache, newe)) {
+        /* if we're doing a modrdn or turning an entry to a tombstone,
+         * the new entry can be in the dn table already, so we need to remove that too.
          */
         if (remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)))
         {
             slapi_counter_subtract(cache->c_cursize, newe->ep_size);
             cache->c_curentries--;
+            newe->ep_refcnt--;
             LOG("entry cache replace remove entry size %lu\n", newe->ep_size, 0, 0);
         }
     }
-
+    /* 
+     * The old entry could have been "removed" between the add and this replace,
+     * The entry is NOT freed, but NOT in the dn hash.
+     * which could happen since the entry is not necessarily locked.
+     * This is ok.
+     */
+    olde->ep_state = ENTRY_STATE_DELETED; /* olde is removed from the cache, so set DELETED here. */
+    if (!found) {
+        if (olde->ep_state & ENTRY_STATE_DELETED) {
+            LOG("entry cache replace (%s): cache index tables out of sync - found dn [%d] id [%d]; but the entry is alreay deleted.\n",
+                oldndn, found_in_dn, found_in_id);
+        } else {
+#ifdef UUIDCACHE_ON
+            LOG("entry cache replace: cache index tables out of sync - found dn [%d] id [%d] uuid [%d]\n",
+                found_in_dn, found_in_id, found_in_uuid);
+#else
+            LOG("entry cache replace (%s): cache index tables out of sync - found dn [%d] id [%d]\n",
+                oldndn, found_in_dn, found_in_id);
+#endif
+            cache_unlock(cache);
+            return 1;
+        }
+    }
     /* now, add the new entry to the hashtables */
     /* (probably don't need such extensive error handling, once this has been
      * tested enough that we believe it works.)
      */
-    if (!add_hash(cache->c_dntable, (void *)newndn, strlen(newndn), newe, NULL)) {
-       LOG("entry cache replace: can't add dn\n", 0, 0, 0);
-       PR_Unlock(cache->c_mutex);
-       return 1;
+    if (!add_hash(cache->c_dntable, (void *)newndn, strlen(newndn), newe, (void **)&alte)) {
+        LOG("entry cache replace (%s): can't add to dn table (returned %s)\n", 
+            newndn, alte?slapi_entry_get_dn(alte->ep_entry):"none", 0);
+        cache_lock(cache);
+        return 1;
     }
-    if (!add_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID), newe, NULL)) {
-       LOG("entry cache replace: can't add id\n", 0, 0, 0);
-       if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
-    	   LOG("entry cache replace: failed to remove dn table\n", 0, 0, 0);
-       }
-       PR_Unlock(cache->c_mutex);
-       return 1;
+    if (!add_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID), newe, (void **)&alte)) {
+        LOG("entry cache replace (%s): can't add to id table (returned %s)\n", 
+            newndn, alte?slapi_entry_get_dn(alte->ep_entry):"none", 0);
+        if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
+            LOG("entry cache replace: failed to remove dn table\n", 0, 0, 0);
+        }
+        cache_unlock(cache);
+        return 1;
     }
 #ifdef UUIDCACHE_ON 
     if (newuuid && !add_hash(cache->c_uuidtable, (void *)newuuid, strlen(newuuid),
                        newe, NULL)) {
-       LOG("entry cache replace: can't add uuid\n", 0, 0, 0);
-       if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
-    	   LOG("entry cache replace: failed to remove dn table(uuid cache)\n", 0, 0, 0);
-       }
-       if(remove_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID)) == 0){
-    	   LOG("entry cache replace: failed to remove id table(uuid cache)\n", 0, 0, 0);
-       }
-       PR_Unlock(cache->c_mutex);
-       return 1;
+        LOG("entry cache replace: can't add uuid\n", 0, 0, 0);
+        if(remove_hash(cache->c_dntable, (void *)newndn, strlen(newndn)) == 0){
+            LOG("entry cache replace: failed to remove dn table(uuid cache)\n", 0, 0, 0);
+        }
+        if(remove_hash(cache->c_idtable, &(newe->ep_id), sizeof(ID)) == 0){
+            LOG("entry cache replace: failed to remove id table(uuid cache)\n", 0, 0, 0);
+        }
+        cache_unlock(cache);
+        return 1;
     }
 #endif
     /* adjust cache meta info */
@@ -1088,9 +1109,8 @@ static int entrycache_replace(struct cache *cache, struct backentry *olde,
     } else if (newe->ep_size < olde->ep_size) {
         slapi_counter_subtract(cache->c_cursize, olde->ep_size - newe->ep_size);
     }
-    olde->ep_state = ENTRY_STATE_DELETED;
     newe->ep_state = 0;
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     LOG("<= entrycache_replace OK,  cache size now %lu cache count now %ld\n",
              slapi_counter_get_value(cache->c_cursize), cache->c_curentries, 0);
     return 0;
@@ -1124,10 +1144,14 @@ entrycache_return(struct cache *cache, struct backentry **bep)
     struct backentry *e;
 
     e = *bep;
+    if (!e) {
+        LDAPDebug0Args(LDAP_DEBUG_ANY, "entrycache_return e is NULL\n");
+        return;
+    }
     LOG("=> entrycache_return (%s) entry count: %d, entry in cache:%ld\n",
                     backentry_get_ndn(e), e->ep_refcnt, cache->c_curentries);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (e->ep_state & ENTRY_STATE_NOTINCACHE)
     {
         backentry_free(bep);
@@ -1146,7 +1170,7 @@ entrycache_return(struct cache *cache, struct backentry **bep)
             }
         }
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     while (eflush)
     {
         eflushtemp = BACK_LRU_NEXT(eflush, struct backentry *);
@@ -1165,22 +1189,22 @@ struct backentry *cache_find_dn(struct cache *cache, const char *dn, unsigned lo
     LOG("=> cache_find_dn (%s)\n", dn, 0, 0);
 
     /*entry normalized by caller (dn2entry.c)  */
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (find_hash(cache->c_dntable, (void *)dn, ndnlen, (void **)&e)) {
        /* need to check entry state */
        if (e->ep_state != 0) {
            /* entry is deleted or not fully created yet */
-           PR_Unlock(cache->c_mutex);
+           cache_unlock(cache);
            LOG("<= cache_find_dn (NOT FOUND)\n", 0, 0, 0);
            return NULL;
        }
        if (e->ep_refcnt == 0)
            lru_delete(cache, (void *)e);
        e->ep_refcnt++;
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        slapi_counter_increment(cache->c_hits);
     } else {
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
     }
     slapi_counter_increment(cache->c_tries);
 
@@ -1196,22 +1220,22 @@ struct backentry *cache_find_id(struct cache *cache, ID id)
 
     LOG("=> cache_find_id (%lu)\n", (u_long)id, 0, 0);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (find_hash(cache->c_idtable, &id, sizeof(ID), (void **)&e)) {
        /* need to check entry state */
        if (e->ep_state != 0) {
            /* entry is deleted or not fully created yet */
-           PR_Unlock(cache->c_mutex);
+           cache_unlock(cache);
            LOG("<= cache_find_id (NOT FOUND)\n", 0, 0, 0);
            return NULL;
        }
        if (e->ep_refcnt == 0)
            lru_delete(cache, (void *)e);
        e->ep_refcnt++;
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        slapi_counter_increment(cache->c_hits);
     } else {
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
     }
     slapi_counter_increment(cache->c_tries);
 
@@ -1227,22 +1251,22 @@ struct backentry *cache_find_uuid(struct cache *cache, const char *uuid)
 
     LOG("=> cache_find_uuid (%s)\n", uuid, 0, 0);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (find_hash(cache->c_uuidtable, uuid, strlen(uuid), (void **)&e)) {
        /* need to check entry state */
        if (e->ep_state != 0) {
            /* entry is deleted or not fully created yet */
-           PR_Unlock(cache->c_mutex);
+           cache_unlock(cache);
            LOG("<= cache_find_uuid (NOT FOUND)\n", 0, 0, 0);
            return NULL;
        }
        if (e->ep_refcnt == 0)
            lru_delete(cache, (void *)e);
        e->ep_refcnt++;
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        slapi_counter_increment(cache->c_hits);
     } else {
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
     }
     slapi_counter_increment(cache->c_tries);
 
@@ -1279,10 +1303,10 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
         entry_size = e->ep_size;
     }
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (! add_hash(cache->c_dntable, (void *)ndn, strlen(ndn), e,
            (void **)&my_alt)) {
-        LOG("entry \"%s\" already in dn cache\n", backentry_get_ndn(e), 0, 0);
+        LOG("entry \"%s\" already in dn cache\n", ndn, 0, 0);
         /* add_hash filled in 'my_alt' if necessary */
         if (my_alt == e)
         {
@@ -1314,7 +1338,7 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                  * to prevent that the caller accidentally thinks the existing
                  * entry is not the same one the caller has and releases it.
                  */
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return 1;
             }
         }
@@ -1322,16 +1346,17 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
         {
             if (my_alt->ep_state & ENTRY_STATE_CREATING)
             {
-                LOG("the entry is reserved\n", 0, 0, 0);
+                LOG("the entry %s is reserved (ep_state: 0x%x, state: 0x%x)\n", ndn, e->ep_state, state);
                 e->ep_state |= ENTRY_STATE_NOTINCACHE;
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return -1;
             }
             else if (state != 0)
             {
-                LOG("the entry already exists. cannot reserve it.\n", 0, 0, 0);
+                LOG("the entry %s already exists. cannot reserve it. (ep_state: 0x%x, state: 0x%x)\n",
+                    ndn, e->ep_state, state);
                 e->ep_state |= ENTRY_STATE_NOTINCACHE;
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return -1;
             }
             else
@@ -1341,9 +1366,16 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                     if ((*alt)->ep_refcnt == 0)
                         lru_delete(cache, (void *)*alt);
                     (*alt)->ep_refcnt++;
+                    LOG("the entry %s already exists.  returning existing entry %s (state: 0x%x)\n",
+                        ndn, backentry_get_ndn(my_alt), state);
+                    cache_unlock(cache);
+                    return 1;
+                } else {
+                    LOG("the entry %s already exists.  Not returning existing entry %s (state: 0x%x)\n",
+                        ndn, backentry_get_ndn(my_alt), state);
+                    cache_unlock(cache);
+                    return -1;
                 }
-                PR_Unlock(cache->c_mutex);
-                return 1;
             }
         }
     }
@@ -1355,7 +1387,7 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
     if (state == 0) {
         /* neither of these should fail, or something is very wrong. */
         if (! add_hash(cache->c_idtable, &(e->ep_id), sizeof(ID), e, NULL)) {
-            LOG("entry %s already in id cache!\n", backentry_get_ndn(e), 0, 0);
+            LOG("entry %s already in id cache!\n", ndn, 0, 0);
             if (already_in) {
                 /* there's a bug in the implementatin of 'modify' and 'modrdn'
                  * that i'm working around here.  basically they do a
@@ -1373,14 +1405,16 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                  * fine (i think).
                  */
                 LOG("<= entrycache_add_int (ignoring)\n", 0, 0, 0);
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return 0;
             }
             if(remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn)) == 0){
-            	LOG("entrycache_add_int: failed to remove dn table\n", 0, 0, 0);
+                LOG("entrycache_add_int: failed to remove %s from dn table\n", 0, 0, 0);
             }
             e->ep_state |= ENTRY_STATE_NOTINCACHE;
-            PR_Unlock(cache->c_mutex);
+            cache_unlock(cache);
+            LOG("entrycache_add_int: failed to add %s to cache (ep_state: %x, already_in: %d)\n",
+                ndn, e->ep_state, already_in);
             return -1;
         }
 #ifdef UUIDCACHE_ON 
@@ -1391,13 +1425,13 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
                 LOG("entry %s already in uuid cache!\n", backentry_get_ndn(e),
                             0, 0);
                 if(remove_hash(cache->c_dntable, (void *)ndn, strlen(ndn)) == 0){
-                	LOG("entrycache_add_int: failed to remove dn table(uuid cache)\n", 0, 0, 0);
+                    LOG("entrycache_add_int: failed to remove dn table(uuid cache)\n", 0, 0, 0);
                 }
                 if(remove_hash(cache->c_idtable, &(e->ep_id), sizeof(ID)) == 0){
-                	LOG("entrycache_add_int: failed to remove id table(uuid cache)\n", 0, 0, 0);
+                    LOG("entrycache_add_int: failed to remove id table(uuid cache)\n", 0, 0, 0);
                 }
                 e->ep_state |= ENTRY_STATE_NOTINCACHE;
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return -1;
             }
         }
@@ -1422,7 +1456,7 @@ entrycache_add_int(struct cache *cache, struct backentry *e, int state,
         if (CACHE_FULL(cache))
             eflush = entrycache_flush(cache);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 
     while (eflush)
     {
@@ -1474,50 +1508,52 @@ int cache_add_tentative(struct cache *cache, struct backentry *e,
 }
 void cache_lock(struct cache *cache)
 {
-    PR_Lock(cache->c_mutex);
+    PR_EnterMonitor(cache->c_mutex);
 }
 void cache_unlock(struct cache *cache)
 {
-    PR_Unlock(cache->c_mutex);
+    PR_ExitMonitor(cache->c_mutex);
 }
 
 /* locks an entry so that it can be modified (you should have gotten the
  * entry via cache_find_*).
- * returns 0 on success, 1 if the entry is scheduled for deletion.
+ * returns 0 on success,
+ * returns 1 if the entry lock could not be created
+ * returns 2 (RETRY_CACHE_LOCK) if the entry is scheduled for deletion.
  */
 int cache_lock_entry(struct cache *cache, struct backentry *e)
 {
     LOG("=> cache_lock_entry (%s)\n", backentry_get_ndn(e), 0, 0);
 
     if (! e->ep_mutexp) {
-       /* make sure only one thread does this */
-       PR_Lock(cache->c_emutexalloc_mutex);
-       if (! e->ep_mutexp) {
-           e->ep_mutexp = PR_NewMonitor();
-           if (!e->ep_mutexp) {
-               LOG("<= cache_lock_entry (DELETED)\n", 0, 0, 0);
-               LDAPDebug1Arg(LDAP_DEBUG_ANY,
-                     "cache_lock_entry: failed to create a lock for %s\n",
-					 backentry_get_ndn(e));
-               LOG("<= cache_lock_entry (FAILED)\n", 0, 0, 0);
-               return 1;
-           }
-       }
-       PR_Unlock(cache->c_emutexalloc_mutex);
+        /* make sure only one thread does this */
+        PR_Lock(cache->c_emutexalloc_mutex);
+        if (! e->ep_mutexp) {
+            e->ep_mutexp = PR_NewMonitor();
+            if (!e->ep_mutexp) {
+                LOG("<= cache_lock_entry (DELETED)\n", 0, 0, 0);
+                LDAPDebug1Arg(LDAP_DEBUG_ANY,
+                              "cache_lock_entry: failed to create a lock for %s\n",
+                              backentry_get_ndn(e));
+                LOG("<= cache_lock_entry (FAILED)\n", 0, 0, 0);
+                return 1;
+            }
+        }
+        PR_Unlock(cache->c_emutexalloc_mutex);
     }
 
     /* wait on entry lock (done w/o holding the cache lock) */
     PR_EnterMonitor(e->ep_mutexp);
 
     /* make sure entry hasn't been deleted now */
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (e->ep_state & (ENTRY_STATE_DELETED|ENTRY_STATE_NOTINCACHE)) {
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        PR_ExitMonitor(e->ep_mutexp);
        LOG("<= cache_lock_entry (DELETED)\n", 0, 0, 0);
-       return 1;
+       return RETRY_CACHE_LOCK;
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 
     LOG("<= cache_lock_entry (FOUND)\n", 0, 0, 0);
     return 0;
@@ -1583,7 +1619,7 @@ dncache_set_max_size(struct cache *cache, size_t bytes)
                 "WARNING -- Minimum cache size is %lu -- rounding up\n",
                 MINCACHESIZE, 0, 0);
     }
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     cache->c_maxsize = bytes;
     LOG("entry cache size set to %lu\n", bytes, 0, 0);
     /* check for full cache, and clear out if necessary */
@@ -1603,7 +1639,7 @@ dncache_set_max_size(struct cache *cache, size_t bytes)
        erase_cache(cache, CACHE_TYPE_DN);
        cache_make_hashes(cache, CACHE_TYPE_DN);
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     if (! dblayer_is_cachesize_sane(&bytes)) {
        LDAPDebug1Arg(LDAP_DEBUG_ANY,
                 "WARNING -- Possible CONFIGURATION ERROR -- cachesize "
@@ -1667,7 +1703,7 @@ dncache_return(struct cache *cache, struct backdn **bdn)
     LOG("=> dncache_return (%s) reference count: %d, dn in cache:%ld\n",
       slapi_sdn_get_dn((*bdn)->dn_sdn), (*bdn)->ep_refcnt, cache->c_curentries);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if ((*bdn)->ep_state & ENTRY_STATE_NOTINCACHE)
     {
         backdn_free(bdn);
@@ -1687,7 +1723,7 @@ dncache_return(struct cache *cache, struct backdn **bdn)
             }
         }
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     while (dnflush)
     {
         dnflushtemp = BACK_LRU_NEXT(dnflush, struct backdn *);
@@ -1708,22 +1744,22 @@ dncache_find_id(struct cache *cache, ID id)
 
     LOG("=> dncache_find_id (%lu)\n", (u_long)id, 0, 0);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
     if (find_hash(cache->c_idtable, &id, sizeof(ID), (void **)&bdn)) {
        /* need to check entry state */
        if (bdn->ep_state != 0) {
            /* entry is deleted or not fully created yet */
-           PR_Unlock(cache->c_mutex);
+           cache_unlock(cache);
            LOG("<= dncache_find_id (NOT FOUND)\n", 0, 0, 0);
            return NULL;
        }
        if (bdn->ep_refcnt == 0)
            lru_delete(cache, (void *)bdn);
        bdn->ep_refcnt++;
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        slapi_counter_increment(cache->c_hits);
     } else {
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
     }
     slapi_counter_increment(cache->c_tries);
 
@@ -1748,7 +1784,7 @@ dncache_add_int(struct cache *cache, struct backdn *bdn, int state,
     LOG("=> dncache_add_int( \"%s\", %ld )\n", slapi_sdn_get_dn(bdn->dn_sdn), 
         bdn->ep_id, 0);
 
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
 
     if (! add_hash(cache->c_idtable, &(bdn->ep_id), sizeof(ID), bdn,
                                                            (void **)&my_alt)) {
@@ -1783,7 +1819,7 @@ dncache_add_int(struct cache *cache, struct backdn *bdn, int state,
                  * to prevent that the caller accidentally thinks the existing
                  * entry is not the same one the caller has and releases it.
                  */
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return 1;
             }
         }
@@ -1793,14 +1829,14 @@ dncache_add_int(struct cache *cache, struct backdn *bdn, int state,
             {
                 LOG("the entry is reserved\n", 0, 0, 0);
                 bdn->ep_state |= ENTRY_STATE_NOTINCACHE;
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return -1;
             }
             else if (state != 0)
             {
                 LOG("the entry already exists. cannot reserve it.\n", 0, 0, 0);
                 bdn->ep_state |= ENTRY_STATE_NOTINCACHE;
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return -1;
             }
             else
@@ -1811,7 +1847,7 @@ dncache_add_int(struct cache *cache, struct backdn *bdn, int state,
                         lru_delete(cache, (void *)*alt);
                     (*alt)->ep_refcnt++;
                 }
-                PR_Unlock(cache->c_mutex);
+                cache_unlock(cache);
                 return 1;
             }
         }
@@ -1840,7 +1876,7 @@ dncache_add_int(struct cache *cache, struct backdn *bdn, int state,
             dnflush = dncache_flush(cache);
         }
     }
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
 
     while (dnflush)
     {
@@ -1868,7 +1904,7 @@ dncache_replace(struct cache *cache, struct backdn *olddn, struct backdn *newdn)
      * where the entry isn't in all the table yet, so we don't care if any
      * of these return errors.
      */
-    PR_Lock(cache->c_mutex);
+    cache_lock(cache);
 
     /*
      * First, remove the old entry from the hashtable.
@@ -1880,7 +1916,7 @@ dncache_replace(struct cache *cache, struct backdn *olddn, struct backdn *newdn)
         found = remove_hash(cache->c_idtable, &(olddn->ep_id), sizeof(ID));
         if (!found) {
             LOG("dn cache replace: cache index tables out of sync\n", 0, 0, 0);
-            PR_Unlock(cache->c_mutex);
+            cache_unlock(cache);
             return 1;
         }
     }
@@ -1891,7 +1927,7 @@ dncache_replace(struct cache *cache, struct backdn *olddn, struct backdn *newdn)
      */
     if (!add_hash(cache->c_idtable, &(newdn->ep_id), sizeof(ID), newdn, NULL)) {
        LOG("dn cache replace: can't add id\n", 0, 0, 0);
-       PR_Unlock(cache->c_mutex);
+       cache_unlock(cache);
        return 1;
     }
     /* adjust cache meta info */
@@ -1906,7 +1942,7 @@ dncache_replace(struct cache *cache, struct backdn *olddn, struct backdn *newdn)
     }
     olddn->ep_state = ENTRY_STATE_DELETED;
     newdn->ep_state = 0;
-    PR_Unlock(cache->c_mutex);
+    cache_unlock(cache);
     LOG("<= dncache_replace OK,  cache size now %lu cache count now %ld\n",
              slapi_counter_get_value(cache->c_cursize), cache->c_curentries, 0);
     return 0;
@@ -1989,3 +2025,94 @@ dn_lru_verify(struct cache *cache, struct backdn *dn, int in)
     ASSERT(is_in == in);
 }
 #endif
+
+#ifdef CACHE_DEBUG
+void
+check_entry_cache(struct cache *cache, struct backentry *e)
+{
+	Slapi_DN *sdn = slapi_entry_get_sdn(e->ep_entry);
+	struct backentry *debug_e = cache_find_dn(cache, 
+	                                          slapi_sdn_get_dn(sdn),
+	                                          slapi_sdn_get_ndn_len(sdn));
+	in_cache = cache_is_in_cache(cache, (void *)e);
+	if (in_cache) {
+		if (debug_e) { /* e is in cache */
+			CACHE_RETURN(cache, &debug_e);
+			if ((e != debug_e) && !(e->ep_state & ENTRY_STATE_DELETED)) {
+				slapi_log_error(SLAPI_LOG_FATAL, "check_entry_cache",
+				                "entry 0x%p is not in dn cache but 0x%p having the same dn %s is "
+				                "although in_cache flag is set!!!\n",
+				                e, debug_e, slapi_sdn_get_dn(sdn));
+			}
+		} else if (!(e->ep_state & ENTRY_STATE_DELETED)) {
+			slapi_log_error(SLAPI_LOG_FATAL, "check_entry_cache",
+			                "%s (id %d) is not in dn cache although in_cache flag is set!!!\n",
+			                slapi_sdn_get_dn(sdn), e->ep_id);
+		}
+		debug_e = cache_find_id(cache, e->ep_id);
+		if (debug_e) { /* e is in cache */
+			CACHE_RETURN(cache, &debug_e);
+			if ((e != debug_e) && !(e->ep_state & ENTRY_STATE_DELETED)) {
+				slapi_log_error(SLAPI_LOG_FATAL, "check_entry_cache",
+				                "entry 0x%p is not in id cache but 0x%p having the same id %d is "
+				                "although in_cache flag is set!!!\n",
+				                e, debug_e, e->ep_id);
+			}
+		} else {
+			slapi_log_error(SLAPI_LOG_CACHE, "check_entry_cache",
+			                "%s (id %d) is not in id cache although in_cache flag is set!!!\n",
+			                slapi_sdn_get_dn(sdn), e->ep_id);
+		}
+	} else {
+		if (debug_e) { /* e is in cache */
+			CACHE_RETURN(cache, &debug_e);
+			if (e == debug_e) {
+				slapi_log_error(SLAPI_LOG_FATAL, "check_entry_cache",
+				                "%s (id %d) is in dn cache although in_cache flag is not set!!!\n",
+				                slapi_sdn_get_dn(sdn), e->ep_id);
+			}
+		}
+		debug_e = cache_find_id(cache, e->ep_id);
+		if (debug_e) { /* e is in cache: bad */
+			CACHE_RETURN(cache, &debug_e);
+			if (e == debug_e) {
+				slapi_log_error(SLAPI_LOG_CACHE, "check_entry_cache",
+				                "%s (id %d) is in id cache although in_cache flag is not set!!!\n",
+				                slapi_sdn_get_dn(sdn), e->ep_id);
+			}
+		}
+	}
+}
+#endif
+
+int
+cache_has_otherref(struct cache *cache, void *ptr)
+{
+    struct backcommon *bep;
+    int hasref = 0;
+
+    if (NULL == ptr) {
+        return hasref;
+    }
+    bep = (struct backcommon *)ptr;
+    /* slows down too much? PR_Lock(cache->c_mutex); */
+    hasref = bep->ep_refcnt;
+    /* PR_Unlock(cache->c_mutex); */
+    return (hasref>1)?1:0;
+}
+
+int
+cache_is_in_cache(struct cache *cache, void *ptr)
+{
+    struct backcommon *bep;
+    int in_cache = 0;
+
+    if (NULL == ptr) {
+        return in_cache;
+    }
+    bep = (struct backcommon *)ptr;
+    /* slows down too much? PR_Lock(cache->c_mutex); */
+    in_cache = (bep->ep_state & (ENTRY_STATE_DELETED|ENTRY_STATE_NOTINCACHE))?0:1;
+    /* PR_Unlock(cache->c_mutex); */
+    return in_cache;
+}
