@@ -61,6 +61,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	struct backentry *e = NULL;
 	struct backentry *tombstone = NULL;
 	struct backentry *original_tombstone = NULL;
+	struct backentry *tmptombstone = NULL;
 	const char *dn = NULL;
 	back_txn txn;
 	back_txnid parent_txn;
@@ -75,7 +76,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	modify_context ruv_c = {0};
 	int rc = 0;
 	int ldap_result_code= LDAP_SUCCESS;
-	char *ldap_result_message= NULL;
+	char *ldap_result_message = NULL;
 	Slapi_DN *sdnp = NULL;
 	char *e_uniqueid = NULL;
 	Slapi_DN nscpEntrySDN;
@@ -88,8 +89,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	int delete_tombstone_entry = 0;	/* We must remove the given tombstone entry from the DB	*/
 	int create_tombstone_entry = 0;	/* We perform a "regular" LDAP delete but since we use	*/
 									/* replication, we must create a new tombstone entry	*/
-	int tombstone_in_cache = 0;
-	int e_in_cache = 0;
+	int remove_e_from_cache = 0;
 	entry_address *addr;
 	int addordel_flags = 0; /* passed to index_addordel */
 	char *entryusn_str = NULL;
@@ -98,6 +98,17 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	int opreturn = 0;
 	int free_delete_existing_entry = 0;
 	int not_an_error = 0;
+	int parent_switched = 0;
+	int myrc = 0;
+	PRUint64 conn_id;
+	int op_id;
+	ID ep_id = 0;
+	ID tomb_ep_id = 0;
+
+	if (slapi_pblock_get(pb, SLAPI_CONN_ID, &conn_id) < 0) {
+		conn_id = 0; /* connection is NULL */
+	}
+	slapi_pblock_get(pb, SLAPI_OPERATION_ID, &op_id);
 
 	slapi_pblock_get( pb, SLAPI_BACKEND, &be);
 	slapi_pblock_get( pb, SLAPI_PLUGIN_PRIVATE, &li );
@@ -108,6 +119,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	slapi_pblock_get( pb, SLAPI_IS_REPLICATED_OPERATION, &is_replicated_operation );
 	
 	slapi_sdn_init(&nscpEntrySDN);
+	slapi_sdn_init(&parentsdn);
 
 	/* dblayer_txn_init needs to be called before "goto error_return" */
 	dblayer_txn_init(li,&txn);
@@ -195,7 +207,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	 */
 	txn.back_txn_txn = NULL; /* ready to create the child transaction */
 	for (retry_count = 0; retry_count < RETRY_TIMES; retry_count++) {
-		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) {
+		if (txn.back_txn_txn && (txn.back_txn_txn != parent_txn)) { /* retry_count > 0 */
 			Slapi_Entry *ent = NULL;
 
 			/* Don't release SERIAL LOCK */
@@ -216,26 +228,30 @@ ldbm_back_delete( Slapi_PBlock *pb )
 
 			/* reset tombstone entry */
 			if (original_tombstone) {
-				if (tombstone_in_cache) {
-					CACHE_REMOVE(&inst->inst_cache, tombstone);
-					CACHE_RETURN(&inst->inst_cache, &tombstone);
-					tombstone_in_cache = 0; 
-				} else {
-					backentry_free(&tombstone);
-				}
-				tombstone = original_tombstone;
-				if ( (original_tombstone = backentry_dup( tombstone )) == NULL ) {
+				/* must duplicate tombstone before returning it to cache,
+				 * which could free the entry. */
+				if ( (tmptombstone = backentry_dup( original_tombstone )) == NULL ) {
 					ldap_result_code= LDAP_OPERATIONS_ERROR;
 					goto error_return;
 				}
+				if (cache_is_in_cache(&inst->inst_cache, tombstone)) {
+					CACHE_REMOVE(&inst->inst_cache, tombstone);
+				}
+				CACHE_RETURN(&inst->inst_cache, &tombstone);
+				if (tombstone) {
+					slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete", 
+					                "conn=%lu op=%d [retry: %d] tombstone %s is not freed!!! refcnt %d, state %d\n",
+					                conn_id, op_id, retry_count, slapi_entry_get_dn(tombstone->ep_entry),
+					                tombstone->ep_refcnt, tombstone->ep_state);
+				}
+				tombstone = original_tombstone;
+				original_tombstone = tmptombstone;
+				tmptombstone = NULL;
+			} else {
+				slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete", 
+				                "conn=%lu op=%d [retry: %d] No original_tombstone for %s!!\n",
+				                conn_id, op_id, retry_count, slapi_entry_get_dn(e->ep_entry));
 			}
-
-			/* reset original entry in cache */
-			if (!e_in_cache) {
-				CACHE_ADD(&inst->inst_cache, e, NULL);
-				e_in_cache = 1;
-			}
-
 			if (ruv_c_init) {
 				/* reset the ruv txn stuff */
 				modify_term(&ruv_c, be);
@@ -243,8 +259,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			}
 
 			/* We're re-trying */
-			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM,
-			               "Delete Retrying Transaction\n");
+			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM, "Delete Retrying Transaction\n");
 #ifndef LDBM_NO_BACKOFF_DELAY
 			{
 				PRIntervalTime interval;
@@ -271,17 +286,28 @@ ldbm_back_delete( Slapi_PBlock *pb )
 
 		if (0 == retry_count) { /* just once */
 			/* find and lock the entry we are about to modify */
+			/* 
+			 * A corner case: 
+			 * If a conflict occurred in a MMR topology, a replicated delete 
+			 * op from another master could target a conflict entry; while the
+			 * corresponding entry on this server could have been already 
+			 * deleted.  That is, the entry 'e' found with "addr" is a tomb-
+			 * stone.  If it is the case, we need to back off.
+			 */
 			if ( (e = find_entry2modify( pb, be, addr, &txn )) == NULL )
 			{
 				ldap_result_code= LDAP_NO_SUCH_OBJECT; 
 				retval = -1;
+				LDAPDebug0Args(LDAP_DEBUG_BACKLDBM, "ldbm_back_delete: Deleting entry is already deleted.\n");
 				goto error_return; /* error result sent by find_entry2modify() */
 			}
-			e_in_cache = 1; /* e is cached */
-		
-			if ( slapi_entry_has_children( e->ep_entry ) )
-			{
+			ep_id = e->ep_id;
+			retval = slapi_entry_has_children(e->ep_entry);
+			if (retval) {
 				ldap_result_code= LDAP_NOT_ALLOWED_ON_NONLEAF;
+				slapi_log_error(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", 
+				                "conn=%lu op=%d Deleting entry %s has %d children.\n", 
+				                conn_id, op_id, slapi_entry_get_dn(e->ep_entry), retval);
 				retval = -1;
 				goto error_return;
 			}
@@ -317,7 +343,17 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					 * Plugin indicated some kind of failure,
 					 * or that this Operation became a No-Op.
 					 */
+					slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
 					if (!ldap_result_code) {
+						if (LDAP_ALREADY_EXISTS == ldap_result_code) {
+							/* 
+							 * The target entry is already a tombstone.
+							 * We need to treat this as a success,
+							 * but we need to remove the entry e from the entry cache.
+							 */
+							remove_e_from_cache = 1;
+							ldap_result_code = LDAP_SUCCESS;
+						}
 						slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
 					}
 					/* restore original entry so the front-end delete code can free it */
@@ -329,8 +365,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					goto error_return;
 				}
 				/* the flag could be set in a preop plugin (e.g., USN) */
-				delete_tombstone_entry = operation_is_flag_set(operation,
-				                                               OP_FLAG_TOMBSTONE_ENTRY);
+				delete_tombstone_entry = operation_is_flag_set(operation, OP_FLAG_TOMBSTONE_ENTRY);
 			}
 
 			/* call the transaction pre delete plugins just after the 
@@ -444,7 +479,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			 * seems to deadlock the database when dblayer_txn_begin is
 			 * called.
 			 */
-			slapi_sdn_init(&parentsdn);
 			slapi_sdn_get_backend_parent_ext(sdnp, &parentsdn, pb->pb_backend, is_tombstone_entry);
 			if ( !slapi_sdn_isempty(&parentsdn) )
 			{
@@ -458,25 +492,50 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					 * (find_entry2modify_only_ext), a wrong parent could be found,
 					 * and numsubordinate count could get confused.
 					 */
-					ID pid = (ID)strtol(pid_str, (char **)NULL, 10);
+					ID pid;
+					int cache_retry_count = 0;
+					int cache_retry = 0;
+
+					pid = (ID)strtol(pid_str, (char **)NULL, 10);
 					slapi_ch_free_string(&pid_str);
-					parent = id2entry(be, pid ,NULL, &retval);
-					if (parent && cache_lock_entry(&inst->inst_cache, parent)) {
-						/* Failed to obtain parent entry's entry lock */
-						CACHE_RETURN(&(inst->inst_cache), &parent);
-						retval = -1;
-						goto error_return;
+
+					/*
+					 * Its possible that the parent entry retrieved from the cache in id2entry
+					 * could be removed before we lock it, because tombstone purging updated/replaced
+					 * the parent.  If we fail to lock the entry, just try again.
+					 */
+					while(1){
+						parent = id2entry(be, pid ,NULL, &retval);
+						if (parent && (cache_retry = cache_lock_entry(&inst->inst_cache, parent))) {
+							/* Failed to obtain parent entry's entry lock */
+							if(cache_retry == RETRY_CACHE_LOCK &&
+							   cache_retry_count < LDBM_CACHE_RETRY_COUNT)
+							{
+								/* try again */
+								DS_Sleep(PR_MillisecondsToInterval(100));
+								cache_retry_count++;
+								continue;
+							}
+							retval = -1;
+							CACHE_RETURN(&(inst->inst_cache), &parent);
+							goto error_return;
+						} else {
+							/* entry locked, move on */
+							break;
+						}
 					}
 				}
 				if (NULL == parent) {
 					entry_address parent_addr;
-
+					if (is_tombstone_entry) {
+						parent_addr.uniqueid = slapi_entry_attr_get_charptr(e->ep_entry, SLAPI_ATTR_VALUE_PARENT_UNIQUEID);
+					} else {
+						parent_addr.uniqueid = NULL;
+					}
 					parent_addr.sdn = &parentsdn;
-					parent_addr.uniqueid = NULL;
-					parent = find_entry2modify_only_ext(pb, be, &parent_addr,
-					                                    TOMBSTONE_INCLUDED, &txn);
+					parent = find_entry2modify_only_ext(pb, be, &parent_addr, TOMBSTONE_INCLUDED, &txn);
 				}
-				if (NULL != parent) {
+				if (parent) {
 					int isglue;
 					size_t haschildren = 0;
 					int op = PARENTUPDATE_DEL;
@@ -493,11 +552,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					} else if (delete_tombstone_entry) {
 						op |= PARENTUPDATE_DELETE_TOMBSTONE;
 					}
-					retval = parent_update_on_childchange(&parent_modify_c,
-					                                      op, &haschildren);
+					retval = parent_update_on_childchange(&parent_modify_c, op, &haschildren);
 					/* The modify context now contains info needed later */
 					if (0 != retval) {
+						slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
+						                "conn=%lu op=%d parent_update_on_childchange: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+						                conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
 						ldap_result_code= LDAP_OPERATIONS_ERROR;
+						slapi_sdn_done(&parentsdn);
 						retval = -1;
 						goto error_return;
 					}
@@ -517,7 +579,6 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					}
 				}
 			}
-			slapi_sdn_done(&parentsdn);
 
 			if(create_tombstone_entry)
 			{
@@ -528,10 +589,20 @@ ldbm_back_delete( Slapi_PBlock *pb )
 				 */
 				const char *childuniqueid= slapi_entry_get_uniqueid(e->ep_entry);
 				const char *parentuniqueid= NULL;
-				char *tombstone_dn = compute_entry_tombstone_dn(slapi_entry_get_dn(e->ep_entry),
-					childuniqueid);
+				char *edn = slapi_entry_get_dn(e->ep_entry);
+				char *tombstone_dn;
 				Slapi_Value *tomb_value;
 
+				if (slapi_is_special_rdn(edn, RDN_IS_TOMBSTONE)) {
+					slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
+					                "conn=%lu op=%d Turning a tombstone into a tombstone! \"%s\"; e: 0x%p, cache_state: 0x%x, refcnt: %d\n", 
+					                conn_id, op_id, edn, e, e->ep_state, e->ep_refcnt);
+					ldap_result_code= LDAP_OPERATIONS_ERROR;
+					retval = -1;
+					goto error_return;
+				}
+				tombstone_dn = compute_entry_tombstone_dn(edn, childuniqueid);
+
 				slapi_sdn_set_ndn_byval(&nscpEntrySDN, slapi_sdn_get_ndn(slapi_entry_get_sdn(e->ep_entry)));
 
 				/* Copy the entry unique_id for URP conflict checking */
@@ -543,6 +614,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					parentuniqueid= slapi_entry_get_uniqueid(parent_modify_c.old_entry->ep_entry);
 				}
 				tombstone = backentry_dup( e );
+				tomb_ep_id = tombstone->ep_id;
 				slapi_entry_set_dn(tombstone->ep_entry,tombstone_dn); /* Consumes DN */
 				if (entryrdn_get_switch()) /* subtree-rename: on */
 				{
@@ -652,41 +724,29 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			 * tentatively for now, then cache_add again when the original
 			 * entry is removed from the cache.
 			 */
-			if (cache_add_tentative( &inst->inst_cache, tombstone, NULL) == 0) {
-				tombstone_in_cache = 1;
-			} else if (!(tombstone->ep_state & ENTRY_STATE_NOTINCACHE)) {
-			    LDAPDebug1Arg(LDAP_DEBUG_CACHE,
-			                  "id2entry_add tombstone (%s) is in cache\n",
-			                  slapi_entry_get_dn(tombstone->ep_entry));
-			    tombstone_in_cache = 1;
-			}
-			retval = id2entry_add( be, tombstone, &txn );
-			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS, "delete 1 DB_LOCK_DEADLOCK\n", 0, 0, 0 );
-				/* Abort and re-try */
-				continue;
-			}
-			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_ANY, "id2entry_add failed, err=%d %s\n",
-					   retval, (msg = dblayer_strerror( retval )) ? msg : "", 0 );
+			retval = cache_add_tentative(&inst->inst_cache, tombstone, NULL);
+			if (0 > retval) {
+				slapi_log_error(SLAPI_LOG_CACHE, "ldbm_back_delete",
+				                "conn=%lu op=%d tombstone entry %s failed to add to the cache: %d\n",
+				                conn_id, op_id, slapi_entry_get_dn(tombstone->ep_entry), retval);
 				if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
 				DEL_SET_ERROR(ldap_result_code, 
 							  LDAP_OPERATIONS_ERROR, retry_count);
 				goto error_return;
 			}
-			if (cache_replace( &inst->inst_cache, e, tombstone ) != 0 ) {
-				LDAPDebug0Args( LDAP_DEBUG_BACKLDBM, "ldbm_back_delete cache_replace failed\n");
-				DEL_SET_ERROR(ldap_result_code, 
-							  LDAP_OPERATIONS_ERROR, retry_count);
-				retval= -1;
+			retval = id2entry_add( be, tombstone, &txn );
+			if (DB_LOCK_DEADLOCK == retval) {
+				LDAPDebug( LDAP_DEBUG_BACKLDBM, "delete 1 DB_LOCK_DEADLOCK\n", 0, 0, 0 );
+				/* Abort and re-try */
+				continue;
+			}
+			if (retval) {
+				LDAPDebug( LDAP_DEBUG_ANY, "id2entry_add failed, err=%d %s\n",
+				           retval, (msg = dblayer_strerror( retval )) ? msg : "", 0 );
+				if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
+				DEL_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 				goto error_return;
-			} else {
-				e_in_cache = 0; /* e un-cached */
 			}
-			/* tombstone was already added to the cache via cache_add_tentative (to reserve its spot in the cache)
-			   and/or id2entry_add - so it already had one refcount - cache_replace adds another refcount -
-			   drop the extra ref added by cache_replace */
-			CACHE_RETURN( &inst->inst_cache, &tombstone );
 		}
 		else
 		{
@@ -694,13 +754,12 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			retval = id2entry_delete( be, e, &txn );
 			if (DB_LOCK_DEADLOCK == retval)
 			{
-				LDAPDebug( LDAP_DEBUG_ARGS, "delete 2 DEADLOCK\n", 0, 0, 0 );
+				LDAPDebug0Args(LDAP_DEBUG_BACKLDBM, "delete 2 DEADLOCK\n");
 				/* Retry txn */
 				continue;
 			}
-			if (retval  != 0 ) {
-			  	if (retval == DB_RUNRECOVERY || 
-				    LDBM_OS_ERR_IS_DISKFULL(retval)) {
+			if (retval) {
+				if (retval == DB_RUNRECOVERY || LDBM_OS_ERR_IS_DISKFULL(retval)) {
 				    disk_full = 1;
 				}
 				DEL_SET_ERROR(ldap_result_code, 
@@ -717,12 +776,13 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		retval = index_addordel_entry( be, e, addordel_flags, &txn );
 		if (DB_LOCK_DEADLOCK == retval)
 		{
-			LDAPDebug( LDAP_DEBUG_ARGS, "delete 1 DEADLOCK\n", 0, 0, 0 );
+			LDAPDebug0Args(LDAP_DEBUG_BACKLDBM, "delete 1 DEADLOCK\n");
 			/* Retry txn */
 			continue;
 		}
-		if (retval != 0) {
-			LDAPDebug( LDAP_DEBUG_TRACE, "index_del_entry failed\n", 0, 0, 0 );
+		if (retval) {
+			LDAPDebug(LDAP_DEBUG_ANY, "index_del_entry(%s, 0x%x) failed (%d)\n", 
+			          slapi_entry_get_dn(e->ep_entry), addordel_flags, retval);
 			DEL_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 			goto error_return;
 		}
@@ -737,14 +797,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							SLAPI_ATTR_VALUE_TOMBSTONE,
 							tombstone->ep_id,BE_INDEX_ADD, &txn);
 			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS,
+				LDAPDebug( LDAP_DEBUG_BACKLDBM,
 							"delete (adding %s) DB_LOCK_DEADLOCK\n",
 							SLAPI_ATTR_VALUE_TOMBSTONE, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
-			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE,
+			if (retval) {
+				LDAPDebug( LDAP_DEBUG_ANY,
 							"delete (adding %s) failed, err=%d %s\n",
 							SLAPI_ATTR_VALUE_TOMBSTONE, retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -757,14 +817,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							slapi_entry_get_uniqueid(tombstone->ep_entry),
 							tombstone->ep_id,BE_INDEX_ADD,&txn);
 			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS,
+				LDAPDebug( LDAP_DEBUG_BACKLDBM,
 							"delete (adding %s) DB_LOCK_DEADLOCK\n",
 							SLAPI_ATTR_UNIQUEID, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
-			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE,
+			if (retval) {
+				LDAPDebug( LDAP_DEBUG_ANY,
 							"delete (adding %s) failed, err=%d %s\n",
 							SLAPI_ATTR_UNIQUEID, retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -777,14 +837,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							slapi_sdn_get_ndn(&nscpEntrySDN),
 							tombstone->ep_id, BE_INDEX_ADD, &txn);
 			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS,
+				LDAPDebug( LDAP_DEBUG_BACKLDBM,
 							"delete (adding %s) DB_LOCK_DEADLOCK\n",
 							SLAPI_ATTR_NSCP_ENTRYDN, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
-			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE, 
+			if (retval) {
+				LDAPDebug( LDAP_DEBUG_ANY, 
 							"delete (adding %s) failed, err=%d %s\n",
 							SLAPI_ATTR_NSCP_ENTRYDN, retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -801,14 +861,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							entryusn_str, tombstone->ep_id, BE_INDEX_ADD, &txn);
 				slapi_ch_free_string(&entryusn_str);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug( LDAP_DEBUG_ARGS,
+					LDAPDebug( LDAP_DEBUG_BACKLDBM,
 								"delete (adding %s) DB_LOCK_DEADLOCK\n",
 								SLAPI_ATTR_ENTRYUSN, 0, 0 );
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
-					LDAPDebug( LDAP_DEBUG_TRACE, 
+					LDAPDebug( LDAP_DEBUG_ANY, 
 								"delete (adding %s) failed, err=%d %s\n",
 								SLAPI_ATTR_ENTRYUSN, retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -832,13 +892,13 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					                                  svals, NULL, e->ep_id, 
 					                                  BE_INDEX_ADD, &txn);
 					if (DB_LOCK_DEADLOCK == retval) {
-						LDAPDebug0Args( LDAP_DEBUG_ARGS,
+						LDAPDebug0Args( LDAP_DEBUG_BACKLDBM,
 										"delete (updating " LDBM_PARENTID_STR ") DB_LOCK_DEADLOCK\n");
 						/* Retry txn */
 						continue;
 					}
 					if ( retval ) {
-						LDAPDebug( LDAP_DEBUG_TRACE, 
+						LDAPDebug( LDAP_DEBUG_ANY, 
 								"delete (deleting %s) failed, err=%d %s\n",
 								LDBM_PARENTID_STR, retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -848,15 +908,16 @@ ldbm_back_delete( Slapi_PBlock *pb )
 						goto error_return;
 					}
 				}
+#if 0 /* The entryrdn element is already deleted in the index_addordel_entry */
 				retval = entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug0Args( LDAP_DEBUG_ARGS,
+					LDAPDebug0Args( LDAP_DEBUG_BACKLDBM,
 								"delete (deleting entryrdn) DB_LOCK_DEADLOCK\n");
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
-					LDAPDebug2Args( LDAP_DEBUG_TRACE, 
+					LDAPDebug2Args( LDAP_DEBUG_ANY, 
 								"delete (deleting entryrdn) failed, err=%d %s\n",
 								retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -865,21 +926,21 @@ ldbm_back_delete( Slapi_PBlock *pb )
 								  LDAP_OPERATIONS_ERROR, retry_count);
 					goto error_return;
 				}
+#endif
 				retval = entryrdn_index_entry(be, tombstone, BE_INDEX_ADD, &txn);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug0Args( LDAP_DEBUG_ARGS,
+					LDAPDebug0Args( LDAP_DEBUG_BACKLDBM,
 								"adding (adding tombstone entryrdn) DB_LOCK_DEADLOCK\n");
 					/* Retry txn */
 					continue;
 				}
-				if (0 != retval) {
-					LDAPDebug2Args( LDAP_DEBUG_TRACE, 
-								"adding (adding tombstone entryrdn) failed, err=%d %s\n",
-								retval,
-								(msg = dblayer_strerror( retval )) ? msg : "" );
+				if (retval) {
+					LDAPDebug(LDAP_DEBUG_ANY, 
+					          "adding (adding tombstone entryrdn %s) failed, err=%d %s\n",
+					          slapi_entry_get_dn(tombstone->ep_entry),
+					          retval, (msg = dblayer_strerror( retval )) ? msg : "" );
 					if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
-					DEL_SET_ERROR(ldap_result_code, 
-								  LDAP_OPERATIONS_ERROR, retry_count);
+					DEL_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 					goto error_return;
 				}
 			}
@@ -896,14 +957,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							SLAPI_ATTR_VALUE_TOMBSTONE, e->ep_id,
 							BE_INDEX_DEL|BE_INDEX_EQUALITY, &txn);
 			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS,
+				LDAPDebug( LDAP_DEBUG_BACKLDBM,
 							"delete (deleting %s) DB_LOCK_DEADLOCK\n",
 							SLAPI_ATTR_VALUE_TOMBSTONE, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
 			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE,
+				LDAPDebug( LDAP_DEBUG_ANY,
 							"delete (deleting %s) failed, err=%d %s\n",
 							SLAPI_ATTR_VALUE_TOMBSTONE, retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -916,14 +977,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							slapi_entry_get_uniqueid(e->ep_entry),
 							e->ep_id, BE_INDEX_DEL|BE_INDEX_EQUALITY, &txn);
 			if (DB_LOCK_DEADLOCK == retval) {
-				LDAPDebug( LDAP_DEBUG_ARGS,
+				LDAPDebug( LDAP_DEBUG_BACKLDBM,
 							"delete (deleting %s) DB_LOCK_DEADLOCK\n",
 							SLAPI_ATTR_UNIQUEID, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
 			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE,
+				LDAPDebug( LDAP_DEBUG_ANY,
 							"delete (deleting %s) failed, err=%d %s\n",
 							SLAPI_ATTR_UNIQUEID, retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -940,14 +1001,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 								nscpedn, e->ep_id, BE_INDEX_DEL|BE_INDEX_EQUALITY, &txn);
 				slapi_ch_free((void **)&nscpedn);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug( LDAP_DEBUG_ARGS,
+					LDAPDebug( LDAP_DEBUG_BACKLDBM,
 								"delete (deleting %s) DB_LOCK_DEADLOCK\n",
 								SLAPI_ATTR_NSCP_ENTRYDN, 0, 0 );
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
-					LDAPDebug( LDAP_DEBUG_TRACE,
+					LDAPDebug( LDAP_DEBUG_ANY,
 								"delete (deleting %s) failed, err=%d %s\n",
 								SLAPI_ATTR_NSCP_ENTRYDN, retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -966,14 +1027,14 @@ ldbm_back_delete( Slapi_PBlock *pb )
 							BE_INDEX_DEL|BE_INDEX_EQUALITY, &txn);
 				slapi_ch_free_string(&entryusn_str);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug( LDAP_DEBUG_ARGS,
+					LDAPDebug( LDAP_DEBUG_BACKLDBM,
 								"delete (deleting %s) DB_LOCK_DEADLOCK\n",
 								SLAPI_ATTR_ENTRYUSN, 0, 0 );
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
-					LDAPDebug( LDAP_DEBUG_TRACE, 
+					LDAPDebug( LDAP_DEBUG_ANY, 
 								"delete (deleting %s) failed, err=%d %s\n",
 								SLAPI_ATTR_ENTRYUSN, retval,
 								(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -987,13 +1048,13 @@ ldbm_back_delete( Slapi_PBlock *pb )
 			{
 				retval = entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
 				if (DB_LOCK_DEADLOCK == retval) {
-					LDAPDebug0Args( LDAP_DEBUG_ARGS,
+					LDAPDebug0Args( LDAP_DEBUG_BACKLDBM,
 							"delete (deleting entryrdn) DB_LOCK_DEADLOCK\n");
 					/* Retry txn */
 					continue;
 				}
 				if (0 != retval) {
-					LDAPDebug2Args( LDAP_DEBUG_TRACE, 
+					LDAPDebug2Args( LDAP_DEBUG_ANY, 
 							"delete (deleting entryrdn) failed, err=%d %s\n",
 							retval,
 							(msg = dblayer_strerror( retval )) ? msg : "" );
@@ -1008,14 +1069,17 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		if (parent_found) {
 			/* Push out the db modifications from the parent entry */
 			retval = modify_update_all(be,pb,&parent_modify_c,&txn);
+			slapi_log_error(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
+			                "conn=%lu op=%d modify_update_all: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+			                conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
 			if (DB_LOCK_DEADLOCK == retval)
 			{
-				LDAPDebug( LDAP_DEBUG_ARGS, "del 4 DEADLOCK\n", 0, 0, 0 );
+				LDAPDebug( LDAP_DEBUG_BACKLDBM, "del 4 DEADLOCK\n", 0, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
 			if (0 != retval) {
-				LDAPDebug( LDAP_DEBUG_TRACE, "delete 3 BAD, err=%d %s\n",
+				LDAPDebug( LDAP_DEBUG_ANY, "delete 3 BAD, err=%d %s\n",
 					   retval, (msg = dblayer_strerror( retval )) ? msg : "", 0 );
 				if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
 				DEL_SET_ERROR(ldap_result_code, 
@@ -1033,7 +1097,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 
 			if (DB_LOCK_DEADLOCK == retval)
 			{
-				LDAPDebug( LDAP_DEBUG_ARGS, "delete DEADLOCK vlv_update_index\n", 0, 0, 0 );
+				LDAPDebug( LDAP_DEBUG_BACKLDBM, "delete DEADLOCK vlv_update_index\n", 0, 0, 0 );
 				/* Retry txn */
 				continue;
 			}
@@ -1086,10 +1150,19 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		retval = -1;
 		goto error_return;
 	}
+    if (create_tombstone_entry) {
+			retval = cache_replace(&inst->inst_cache, e, tombstone);
+			if (retval) {
+				LDAPDebug(LDAP_DEBUG_CACHE, "ldbm_back_delete: cache_replace failed (%d): %s --> %s\n",
+				          retval, slapi_entry_get_dn(e->ep_entry), slapi_entry_get_dn(tombstone->ep_entry));
+				retval= -1;
+				goto error_return;
+			} 
+    }
 
 	/* call the transaction post delete plugins just before the commit */
 	if (plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN)) {
-		LDAPDebug0Args( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
+		LDAPDebug0Args( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
 						"returned error code\n" );
 		if (!ldap_result_code) {
 			slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
@@ -1126,20 +1199,28 @@ ldbm_back_delete( Slapi_PBlock *pb )
 
 	/* delete from cache and clean up */
 	if (e) {
-		if (e_in_cache) {
+		if (cache_is_in_cache(&inst->inst_cache, e)) {
+			ep_id = e->ep_id; /* Otherwise, e might have been freed. */
 			CACHE_REMOVE(&inst->inst_cache, e);
 		}
 		cache_unlock_entry(&inst->inst_cache, e);
 		CACHE_RETURN(&inst->inst_cache, &e);
+		/* 
+		 * e is unlocked and no longer in cache.
+		 * It could be freed at any moment.
+		 */
 		e = NULL;
+
+		if (entryrdn_get_switch() && ep_id) { /* subtree-rename: on */
+			/* since the op was successful, delete the tombstone dn from the dn cache */
+			struct backdn *bdn = dncache_find_id(&inst->inst_dncache, ep_id);
+			if (bdn) { /* in the dncache, remove it. */
+				CACHE_REMOVE(&inst->inst_dncache, bdn);
+				CACHE_RETURN(&inst->inst_dncache, &bdn);
+			}
+		}
 	}
 
-	if (parent_found)
-	{
-		 /* Replace the old parent entry with the newly modified one */
-		modify_switch_entries( &parent_modify_c,be);
-	}
-	
 	if (ruv_c_init) {
 		if (modify_switch_entries(&ruv_c, be) != 0 ) {
 			ldap_result_code= LDAP_OPERATIONS_ERROR;
@@ -1150,22 +1231,40 @@ ldbm_back_delete( Slapi_PBlock *pb )
 		}
 	}
 
+	if (parent_found)
+	{
+		/* Replace the old parent entry with the newly modified one */
+		myrc = modify_switch_entries( &parent_modify_c,be);
+		slapi_log_error(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
+		                "conn=%lu op=%d modify_switch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+		                conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, myrc);
+		if (myrc == 0) {
+			parent_switched = 1;
+		}
+	}
+
 	rc= 0;
 	goto common_return;
 
 error_return:
-	if (inst && tombstone_in_cache)
-	{
-		CACHE_REMOVE( &inst->inst_cache, tombstone );
+	if (tombstone) {
+		if (cache_is_in_cache(&inst->inst_cache, tombstone)) {
+			tomb_ep_id = tombstone->ep_id; /* Otherwise, tombstone might have been freed. */
+		}
+		if (entryrdn_get_switch() && tomb_ep_id) { /* subtree-rename: on */
+			struct backdn *bdn = dncache_find_id(&inst->inst_dncache, tombstone->ep_id);
+			if (bdn) { /* already in the dncache. Delete it. */
+				CACHE_REMOVE(&inst->inst_dncache, bdn);
+				CACHE_RETURN(&inst->inst_dncache, &bdn);
+			} 
+		}
+
+		if (cache_is_in_cache(&inst->inst_cache, tombstone)) {
+			CACHE_REMOVE(&inst->inst_cache, tombstone);
+		}
 		CACHE_RETURN( &inst->inst_cache, &tombstone );
 		tombstone = NULL;
-		tombstone_in_cache = 0;
 	}
-	else
-	{
-		backentry_free( &tombstone );
-	}
-
 	if (retval == DB_RUNRECOVERY) {
 	    dblayer_remember_disk_filled(li);
 	    ldbm_nasty("Delete",79,retval);
@@ -1198,7 +1297,7 @@ error_return:
 
 		/* call the transaction post delete plugins just before the abort */
 		if (plugin_call_plugins(pb, SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN)) {
-			LDAPDebug1Arg( LDAP_DEBUG_ANY, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
+			LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN plugin "
 						   "returned error code %d\n", retval );
 			if (!ldap_result_code) {
 				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
@@ -1209,6 +1308,7 @@ error_return:
 			if (!opreturn) {
 				slapi_pblock_set( pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval );
 			}
+			slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
 		}
 
 		/* Release SERIAL LOCK */
@@ -1216,7 +1316,17 @@ error_return:
 		/* txn is no longer valid - reset the txn pointer to the parent */
 		slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
 	}
-	
+	if (parent_switched) {
+		/*
+		 * Restore the old parent entry, switch the new with the original.
+		 * Otherwise the numsubordinate count will be off, and could later
+		 * be written to disk.
+		 */
+		myrc = modify_unswitch_entries(&parent_modify_c, be);
+		slapi_log_error(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
+		                "conn=%lu op=%d modify_unswitch_entries: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
+		                conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, myrc);
+	}
 common_return:
 	if (orig_entry) {
 		/* NOTE: #define SLAPI_DELETE_BEPREOP_ENTRY SLAPI_ENTRY_PRE_OP */
@@ -1224,15 +1334,28 @@ common_return:
 		   for the post op plugins */
 		slapi_pblock_set( pb, SLAPI_DELETE_BEPREOP_ENTRY, orig_entry );
 	}
-	if (inst && tombstone_in_cache)
-	{
+	if (inst && tombstone) {
+		if ((0 == retval) && entryrdn_get_switch()) { /* subtree-rename: on */
+			/* since the op was successful, add the addingentry's dn to the dn cache */
+			struct backdn *bdn = dncache_find_id(&inst->inst_dncache,
+			                                     tombstone->ep_id);
+			if (bdn) { /* already in the dncache */
+				CACHE_RETURN(&inst->inst_dncache, &bdn);
+			} else { /* not in the dncache yet */
+				Slapi_DN *tombstonesdn = slapi_sdn_dup(slapi_entry_get_sdn(tombstone->ep_entry));
+				if (tombstonesdn) {
+					bdn = backdn_init(tombstonesdn, tombstone->ep_id, 0);
+					if (bdn) {
+						CACHE_ADD( &inst->inst_dncache, bdn, NULL );
+						slapi_log_error(SLAPI_LOG_CACHE, "ldbm_back_delete",
+						                "set %s to dn cache\n", slapi_sdn_get_dn(tombstonesdn));
+						CACHE_RETURN(&inst->inst_dncache, &bdn);
+					}
+				}
+			}
+		}
 		CACHE_RETURN( &inst->inst_cache, &tombstone );
 		tombstone = NULL;
-		tombstone_in_cache = 0;
-	}
-	else
-	{
-		backentry_free( &tombstone );
 	}
 	
 	/* result code could be used in the bepost plugin functions. */
@@ -1244,18 +1367,25 @@ common_return:
 	if (!delete_tombstone_entry) {
 		plugin_call_plugins (pb, SLAPI_PLUGIN_BE_POST_DELETE_FN);
 	}
-
 	/* Need to return to cache after post op plugins are called */
-	if (inst) {
-		if (retval && e) { /* error case */
-			cache_unlock_entry( &inst->inst_cache, e );
-			CACHE_RETURN( &inst->inst_cache, &e );
-		}
-		if (inst->inst_ref_count) {
-			slapi_counter_decrement(inst->inst_ref_count);
+	if (e) {
+		if (cache_is_in_cache(&inst->inst_cache, e)) {
+			if (remove_e_from_cache) {
+				/* The entry is already transformed to a tombstone. */
+				CACHE_REMOVE( &inst->inst_cache, e );
+			}
 		}
+		cache_unlock_entry( &inst->inst_cache, e );
+		CACHE_RETURN( &inst->inst_cache, &e );
+		/* 
+		 * e is unlocked and no longer in cache.
+		 * It could be freed at any moment.
+		 */
+		e = NULL;
+	}
+	if (inst && inst->inst_ref_count) { 
+		slapi_counter_decrement(inst->inst_ref_count); 
 	}
-	
 	if (ruv_c_init) {
 		modify_term(&ruv_c, be);
 	}
@@ -1269,7 +1399,11 @@ diskfull_return:
 		}
 		slapi_send_ldap_result( pb, ldap_result_code, NULL, ldap_result_message, 0, NULL );
 	}
-	modify_term(&parent_modify_c,be);
+	slapi_log_error(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
+	                "conn=%lu op=%d modify_term: old_entry=0x%p, new_entry=0x%p, in_cache=%d\n",
+	                conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry,
+	                cache_is_in_cache(&inst->inst_cache, parent_modify_c.new_entry));
+	myrc = modify_term(&parent_modify_c, be);
 	if (rc == 0 && opcsn && !is_fixup_operation && !delete_tombstone_entry)
 	{
 		/* URP Naming Collision
@@ -1286,13 +1420,16 @@ diskfull_return:
 		slapi_pblock_set(pb, SLAPI_DELETE_EXISTING_ENTRY, NULL);
 	}
 	backentry_free(&original_tombstone);
+	backentry_free(&tmptombstone);
 	slapi_ch_free((void**)&errbuf);
 	slapi_sdn_done(&nscpEntrySDN);
 	slapi_ch_free_string(&e_uniqueid);
+	slapi_sdn_done(&parentsdn);
 	if (pb->pb_conn)
 	{
 		slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_delete", "leave conn=%" NSPRIu64 " op=%d\n",
 				(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid);
 	}
+
 	return rc;
 }
