@@ -65,7 +65,6 @@ void modify_init(modify_context *mc,struct backentry *old_entry)
 	PR_ASSERT(NULL == mc->new_entry);
 
 	mc->old_entry = old_entry;
-	mc->new_entry_in_cache = 0;
 	mc->attr_encrypt = 1;
 }
 
@@ -86,6 +85,9 @@ int modify_apply_mods_ignore_error(modify_context *mc, Slapi_Mods *smods, int er
 		ret = entry_apply_mods_ignore_error( mc->new_entry->ep_entry, slapi_mods_get_ldapmods_byref(smods), error);
 	}
 	mc->smods= smods;
+	if (error == ret) {
+		ret = LDAP_SUCCESS;
+	}
 	return ret;
 }
 
@@ -96,16 +98,13 @@ int modify_term(modify_context *mc,struct backend *be)
 
     slapi_mods_free(&mc->smods);
     /* Unlock and return entries */
-    if (NULL != mc->old_entry)  {
+    if (mc->old_entry)  {
         cache_unlock_entry(&inst->inst_cache, mc->old_entry);
         CACHE_RETURN( &(inst->inst_cache), &(mc->old_entry) );
         mc->old_entry= NULL;
     }
-    if (mc->new_entry_in_cache) {
-        CACHE_RETURN( &(inst->inst_cache), &(mc->new_entry) );
-    } else {
-        backentry_free(&(mc->new_entry));
-    }
+
+    CACHE_RETURN(&(inst->inst_cache), &(mc->new_entry));
     mc->new_entry= NULL;
     return 0;
 }
@@ -115,13 +114,63 @@ int modify_switch_entries(modify_context *mc,backend *be)
 {
 	ldbm_instance *inst = (ldbm_instance *) be->be_instance_info;
 	int ret = 0;
-	if (mc->old_entry!=NULL && mc->new_entry!=NULL) {
-	    ret = cache_replace(&(inst->inst_cache), mc->old_entry, mc->new_entry);
-            if (ret == 0) mc->new_entry_in_cache = 1;
+	if (mc->old_entry && mc->new_entry) {
+		ret = cache_replace(&(inst->inst_cache), mc->old_entry, mc->new_entry);
+		if (ret) {
+			LDAPDebug(LDAP_DEBUG_CACHE, "modify_switch_entries: replacing %s with %s failed (%d)\n",
+			          slapi_entry_get_dn(mc->old_entry->ep_entry), 
+			          slapi_entry_get_dn(mc->new_entry->ep_entry), ret);
+		}
 	}
 	return ret;
 }
 
+/*
+ * Switch the new with the old(original) - undoing modify_switch_entries()
+ * This expects modify_term() to be called next, as the old "new" entry
+ * is now gone(replaced by the original entry).
+ */
+int
+modify_unswitch_entries(modify_context *mc,backend *be)
+{
+	struct backentry *tmp_be;
+	ldbm_instance *inst = (ldbm_instance *) be->be_instance_info;
+	int ret = 0;
+
+	if (mc->old_entry && mc->new_entry && 
+	    cache_is_in_cache(&inst->inst_cache, mc->new_entry)) {
+		/* switch the entries, and reset the new, new, entry */
+		tmp_be = mc->new_entry;
+		mc->new_entry = mc->old_entry;
+		mc->new_entry->ep_state = 0;
+		if (cache_has_otherref(&(inst->inst_cache), mc->new_entry)) {
+			/* some other thread refers the entry */
+			CACHE_RETURN(&(inst->inst_cache), &(mc->new_entry));
+		} else {
+			/* don't call CACHE_RETURN, that frees the entry!  */
+			mc->new_entry->ep_refcnt = 0;
+		}
+		mc->old_entry = tmp_be;
+
+		ret = cache_replace(&(inst->inst_cache), mc->old_entry, mc->new_entry);
+		if (ret == 0) {
+			/*
+			 * The new entry was originally locked, so since we did the
+			 * switch we need to unlock the "new" entry, and return the
+			 * "old" one.  modify_term() will then return the "new" entry.
+			 */
+			cache_unlock_entry(&inst->inst_cache, mc->new_entry);
+			cache_lock_entry(&inst->inst_cache, mc->old_entry);
+		} else {
+			LDAPDebug(LDAP_DEBUG_CACHE, "modify_unswitch_entries: replacing %s with %s failed (%d)\n",
+			          slapi_entry_get_dn(mc->old_entry->ep_entry), 
+			          slapi_entry_get_dn(mc->new_entry->ep_entry), ret);
+		}
+	}
+
+	return ret;
+}
+
 /* This routine does that part of a modify operation which involves
    updating the on-disk data: updates idices, id2entry. 
    Copes properly with DB_LOCK_DEADLOCK. The caller must be able to cope with 
@@ -253,7 +302,7 @@ modify_apply_check_expand(
 	 * If the objectClass attribute type was modified in any way, expand
 	 * the objectClass values to reflect the inheritance hierarchy.
 	 */
-	for ( i = 0; mods[i] != NULL && !repl_op; ++i ) {
+	for ( i = 0; (mods != NULL) && (mods[i] != NULL) && !repl_op; ++i ) {
 		if ( 0 == strcasecmp( SLAPI_ATTR_OBJECTCLASS, mods[i]->mod_type )) {
 			slapi_schema_expand_objectclasses( ec->ep_entry );
 			break;
@@ -343,7 +392,8 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	backend *be;
 	ldbm_instance *inst = NULL;
 	struct ldbminfo		*li;
-	struct backentry	*e = NULL, *ec = NULL, *original_entry = NULL;
+	struct backentry	*e = NULL, *ec = NULL;
+	struct backentry	*original_entry = NULL, *tmpentry = NULL;
 	Slapi_Entry		*postentry = NULL;
 	LDAPMod			**mods = NULL;
 	LDAPMod			**mods_original = NULL;
@@ -362,7 +412,6 @@ ldbm_back_modify( Slapi_PBlock *pb )
 	int rc = 0;
 	Slapi_Operation *operation;
 	entry_address *addr;
-	int ec_in_cache = 0;
 	int is_fixup_operation= 0;
 	int is_ruv = 0;                 /* True if the current entry is RUV */
 	CSN *opcsn = NULL;
@@ -464,14 +513,22 @@ ldbm_back_modify( Slapi_PBlock *pb )
 			slapi_pblock_get(pb, SLAPI_MODIFY_MODS, &mods);
 			ldap_mods_free(mods, 1);
 			slapi_pblock_set(pb, SLAPI_MODIFY_MODS, copy_mods(mods_original));
-			/* ec is not really added to the cache until cache_replace, so we
-			   don't have to worry about the cache here */
-			backentry_free(&ec);
-			slapi_pblock_set( pb, SLAPI_MODIFY_EXISTING_ENTRY, original_entry->ep_entry );
-			ec = original_entry;
-			if ( (original_entry = backentry_dup( ec )) == NULL ) {
-				ldap_result_code= LDAP_OPERATIONS_ERROR;
-				goto error_return;
+
+			/* reset ec set cache in id2entry_add_ext */
+			if (ec) {
+				/* must duplicate ec before returning it to cache,
+				 * which could free the entry. */
+				if ( (tmpentry = backentry_dup( ec )) == NULL ) {
+					ldap_result_code= LDAP_OPERATIONS_ERROR;
+					goto error_return;
+				}
+				if (cache_is_in_cache(&inst->inst_cache, ec)) {
+					CACHE_REMOVE(&inst->inst_cache, ec);
+				}
+				CACHE_RETURN(&inst->inst_cache, &ec);
+				ec = original_entry;
+				original_entry = tmpentry;
+				tmpentry = NULL;
 			}
 
 			if (ruv_c_init) {
@@ -520,7 +577,7 @@ ldbm_back_modify( Slapi_PBlock *pb )
 		
 			if ( !is_fixup_operation )
 			{
-				if (slapi_entry_flag_is_set(e->ep_entry, SLAPI_ENTRY_FLAG_TOMBSTONE) ) {
+				if (!repl_op && slapi_entry_flag_is_set(e->ep_entry, SLAPI_ENTRY_FLAG_TOMBSTONE) ) {
 					ldap_result_code = LDAP_UNWILLING_TO_PERFORM;
                 			ldap_result_message = "Operation not allowed on tombstone entry.";
 					slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_modify",
@@ -757,20 +814,19 @@ ldbm_back_modify( Slapi_PBlock *pb )
 		MOD_SET_ERROR(ldap_result_code, LDAP_OPERATIONS_ERROR, retry_count);
 		goto error_return;
 	}
+	/* e uncached */
+	/* we must return both e (which has been deleted) and new entry ec to cache */
+	/* cache_replace removes e from the cache hash tables */
+	cache_unlock_entry( &inst->inst_cache, e );
+	CACHE_RETURN( &inst->inst_cache, &e );
 	/* lock new entry in cache to prevent usage until we are complete */
 	cache_lock_entry( &inst->inst_cache, ec );
-	ec_in_cache = 1;
-
 	postentry = slapi_entry_dup( ec->ep_entry );
 	slapi_pblock_set( pb, SLAPI_ENTRY_POST_OP, postentry );
 
 	/* invalidate virtual cache */
 	ec->ep_entry->e_virtual_watermark = 0;
 
-	/* we must return both e (which has been deleted) and new entry ec */
-	/* cache_replace removes e from the caches */
-	cache_unlock_entry( &inst->inst_cache, e );
-	CACHE_RETURN( &inst->inst_cache, &e );
 	/* 
 	 * LP Fix of crash when the commit will fail:
 	 * If the commit fail, the common error path will
@@ -851,6 +907,7 @@ error_return:
 				LDAPDebug1Arg( LDAP_DEBUG_TRACE, "SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN plugin "
 							   "returned error code %d\n", retval );
 				slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
+				slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
 				slapi_pblock_get(pb, SLAPI_PLUGIN_OPRETURN, &opreturn);
 				if (!opreturn) {
 					slapi_pblock_set(pb, SLAPI_PLUGIN_OPRETURN, ldap_result_code ? &ldap_result_code : &retval);
@@ -869,33 +926,30 @@ error_return:
 	}
 
 	/* if ec is in cache, remove it, then add back e if we still have it */
-	if (inst && ec_in_cache) {
+	if (inst && cache_is_in_cache(&inst->inst_cache, ec)) {
 		CACHE_REMOVE( &inst->inst_cache, ec );
 		/* if ec was in cache, e was not - add back e */
 		if (e) {
-			CACHE_ADD( &inst->inst_cache, e, NULL );
-			cache_lock_entry( &inst->inst_cache, e );
+			if (CACHE_ADD( &inst->inst_cache, e, NULL ) < 0) {
+				LDAPDebug1Arg(LDAP_DEBUG_CACHE, "ldbm_modify: CACHE_ADD %s failed\n",
+				              slapi_entry_get_dn(e->ep_entry));
+			}
 		}
 	}
 
 common_return:
 	slapi_mods_done(&smods);
 	
-	if (inst && ec_in_cache)
-	{
-		cache_unlock_entry( &inst->inst_cache, ec);
-		CACHE_RETURN( &inst->inst_cache, &ec );
-	}
-	else
-	{
-		backentry_free(&ec);
-	}
-	
 	if (inst) {
-		if (e) {
-			cache_unlock_entry( &inst->inst_cache, e);
-			CACHE_RETURN( &inst->inst_cache, &e);
+		if (cache_is_in_cache(&inst->inst_cache, ec)) {
+			cache_unlock_entry(&inst->inst_cache, ec);
+		} else if (e) {
+			/* if ec was not in cache, cache_replace was not done.
+			 * i.e., e was not unlocked. */
+			cache_unlock_entry(&inst->inst_cache, e);
+			CACHE_RETURN(&inst->inst_cache, &e);
 		}
+		CACHE_RETURN(&inst->inst_cache, &ec);
 		if (inst->inst_ref_count) {
 			slapi_counter_decrement(inst->inst_ref_count);
 		}
@@ -925,6 +979,7 @@ common_return:
 	/* free our backups */
 	ldap_mods_free(mods_original, 1);
 	backentry_free(&original_entry);
+	backentry_free(&tmpentry);
 	slapi_ch_free_string(&errbuf);
 
 	return rc;
@@ -938,7 +993,7 @@ remove_illegal_mods(LDAPMod **mods)
 	LDAPMod		*tmp;
 
 	/* remove any attempts by the user to modify these attrs */
-	for ( i = 0; mods[i] != NULL; i++ ) {
+	for ( i = 0; (mods != NULL) && (mods[i] != NULL); i++ ) {
 		if ( strcasecmp( mods[i]->mod_type, numsubordinates ) == 0
 		    || strcasecmp( mods[i]->mod_type, hassubordinates ) == 0 )
 		{
