@@ -242,7 +242,7 @@ ldbm_back_seq( Slapi_PBlock *pb )
 				key.flags = 0;
 				for (retry_count = 0; retry_count < IDL_FETCH_RETRY_COUNT; retry_count++) {
 					err = NEW_IDL_DEFAULT;
-					idl_free(idl);
+					idl_free(&idl);
 					idl = idl_fetch( be, db, &key, txn.back_txn_txn, ai, &err );
 					if(err == DB_LOCK_DEADLOCK) {
 						ldbm_nasty("ldbm_back_seq deadlock retry", 1600, err);
@@ -297,7 +297,7 @@ ldbm_back_seq( Slapi_PBlock *pb )
 		    }
 		    CACHE_RETURN( &inst->inst_cache, &e );
 		}
-		idl_free( idl );
+		idl_free( &idl );
 	}
 
 	dblayer_release_index_file( be, ai, db );
