@@ -521,7 +521,7 @@ static int ids_sasl_getpluginpath(sasl_conn_t *conn, const char **path)
      * 64-bit Linux machines, and /usr/lib/sasl2 on all other platforms.
      */
     char *pluginpath = config_get_saslpath();
-    if ((!pluginpath) || (*pluginpath == '\0')) {
+    if (!pluginpath) {
         if (!(pluginpath = getenv("SASL_PATH"))) {
 #if defined(LINUX) && defined(__LP64__)
             pluginpath = "/usr/lib64/sasl2";
