@@ -404,43 +404,47 @@ char *slapi_filter_to_string_internal( const struct slapi_filter *f, char *buf,
 
 /* operation.c */
 
-#define OP_FLAG_PS                       0x00001
-#define OP_FLAG_PS_CHANGESONLY           0x00002
-#define OP_FLAG_GET_EFFECTIVE_RIGHTS     0x00004  
-#define OP_FLAG_REPLICATED               0x00008 /* A Replicated Operation */
-#define OP_FLAG_REPL_FIXUP               0x00010 /* A Fixup Operation, 
+#define OP_FLAG_PS                       0x000001
+#define OP_FLAG_PS_CHANGESONLY           0x000002
+#define OP_FLAG_GET_EFFECTIVE_RIGHTS     0x000004
+#define OP_FLAG_REPLICATED               0x000008 /* A Replicated Operation */
+#define OP_FLAG_REPL_FIXUP               0x000010 /* A Fixup Operation,
                                                   * generated as a consequence
                                                   * of a Replicated Operation.
                                                   */
-#define OP_FLAG_INTERNAL                 SLAPI_OP_FLAG_INTERNAL  /* 0x00020 */ 
-#define OP_FLAG_ACTION_LOG_ACCESS        0x00040
-#define OP_FLAG_ACTION_LOG_AUDIT         0x00080
-#define OP_FLAG_ACTION_SCHEMA_CHECK      0x00100
-#define OP_FLAG_ACTION_LOG_CHANGES       0x00200
-#define OP_FLAG_ACTION_INVOKE_FOR_REPLOP 0x00400
-#define OP_FLAG_NEVER_CHAIN              SLAPI_OP_FLAG_NEVER_CHAIN  /* 0x0800 */
-#define OP_FLAG_TOMBSTONE_ENTRY          0x01000
-#define OP_FLAG_RESURECT_ENTRY           0x02000
-#define OP_FLAG_LEGACY_REPLICATION_DN    0x04000 /* Operation done by legacy 
+#define OP_FLAG_INTERNAL                 SLAPI_OP_FLAG_INTERNAL  /* 0x000020 */
+#define OP_FLAG_ACTION_LOG_ACCESS        0x000040
+#define OP_FLAG_ACTION_LOG_AUDIT         0x000080
+#define OP_FLAG_ACTION_SCHEMA_CHECK      0x000100
+#define OP_FLAG_ACTION_LOG_CHANGES       0x000200
+#define OP_FLAG_ACTION_INVOKE_FOR_REPLOP 0x000400
+#define OP_FLAG_NEVER_CHAIN              SLAPI_OP_FLAG_NEVER_CHAIN  /* 0x000800 */
+#define OP_FLAG_TOMBSTONE_ENTRY          0x001000
+#define OP_FLAG_RESURECT_ENTRY           0x002000
+#define OP_FLAG_LEGACY_REPLICATION_DN    0x004000 /* Operation done by legacy
                                                   * replication DN
                                                   */
-#define OP_FLAG_ACTION_NOLOG             0x08000 /* Do not log the entry in 
+#define OP_FLAG_ACTION_NOLOG             0x008000 /* Do not log the entry in
                                                   * audit log or change log
                                                   */
-#define OP_FLAG_SKIP_MODIFIED_ATTRS      0x10000 /* Do not update the 
+#define OP_FLAG_SKIP_MODIFIED_ATTRS      0x010000 /* Do not update the
                                                   * modifiersname,
                                                   * modifiedtimestamp, etc.
                                                   * attributes
                                                   */
-#define OP_FLAG_REPL_RUV                 0x20000 /* Flag to tell to the backend
+#define OP_FLAG_REPL_RUV                 0x020000 /* Flag to tell to the backend
                                                   * that the entry to be added/
                                                   * modified is RUV. This info
                                                   * is used to skip VLV op.
                                                   * (see #329951)
                                                   */
-#define OP_FLAG_PAGED_RESULTS            0x40000 /* simple paged results */
-#define OP_FLAG_SERVER_SIDE_SORTING      0x80000 /* server side sorting  */
+#define OP_FLAG_PAGED_RESULTS            0x040000 /* simple paged results */
+#define OP_FLAG_SERVER_SIDE_SORTING      0x080000 /* server side sorting  */
+#define OP_FLAG_REVERSE_CANDIDATE_ORDER  0x100000 /* reverse the search candidate list */
 
+/* reverse search states */
+#define REV_STARTED 1
+#define LAST_REV_ENTRY 2
 
 CSN *operation_get_csn(Slapi_Operation *op);
 void operation_set_csn(Slapi_Operation *op,CSN *csn);
@@ -754,6 +758,9 @@ struct slapi_componentid * plugin_get_default_component_id();
 #define SLAPI_COMPONENT_NAME_NSPR	"Netscape Portable Runtime"
 #define SLAPI_COMPONENT_NAME_LDAPSDK	"LDAP sdk"
 
+/* loads the policies related to the replication of the schema */
+int slapi_schema_load_repl_policies();
+void slapi_schema_get_repl_entries(char **repl_schema_top, char ** repl_schema_supplier, char **repl_schema_consumer, char **default_supplier_policy, char **default_consumer_policy);
 /* return the list of attr defined in the schema matching the attr flags */
 char ** slapi_schema_list_attribute_names(unsigned long flag);
 /* return the list of attributes belonging to the objectclass */
@@ -1011,6 +1018,13 @@ int slapi_uniqueIDIsUUID(const Slapi_UniqueID *uId);
  */
 int slapi_uniqueIDSize( void );
 
+/* Name:		slapi_uniqueIDRdnSize
+   Description:	returns size of SLAPI_ATTR_UNIQUEID=slapi_uniqueIDSize()
+   Parameters:  none
+   Return:		size of the string version of "SLAPI_ATTR_UNIQUEID=uniqueID" in bytes
+ */
+int slapi_uniqueIDRdnSize( void );
+
 /* Name:		slapi_uniqueIDDup
    Description:	duplicates an UniqueID object
    Parameters:	uId - id to duplicate
@@ -1258,6 +1272,40 @@ void    DS_Sleep(PRIntervalTime ticks);
 /* plugin.c */
 int plugin_enabled(const char *plugin_name, void *identity);
 
+/**
+ * For "database" plugins that need to call preoperation backend & backend txn plugins.
+ * This function should be called right before the operation is performed.
+ *
+ * \param Slapi_PBLock object
+ * \param int operation
+ *
+ *     Operations:
+ *         SLAPI_PLUGIN_ADD_OP
+ *         SLAPI_PLUGIN_MOD_OP
+ *         SLAPI_PLUGIN_MODRDN_OP
+ *         SLAPI_PLUGIN_DEL_OP
+ *
+ * \return zero on success, non-zero for failure
+ */
+int slapi_plugin_call_preop_be_plugins(Slapi_PBlock *pb, int operation);
+
+/**
+ * For "database" plugins that need to call postoperation backend & backend txn plugins.
+ * This function should be called right after the operation is performed.
+ *
+ * \param Slapi_PBLock object
+ * \param int operation
+ *
+ *     Operations:
+ *         SLAPI_PLUGIN_ADD_OP
+ *         SLAPI_PLUGIN_MOD_OP
+ *         SLAPI_PLUGIN_MODRDN_OP
+ *         SLAPI_PLUGIN_DEL_OP
+ *
+ * \return zero on success, non-zero for failure
+ */
+int slapi_plugin_call_postop_be_plugins(Slapi_PBlock *pb, int operation);
+
 /* protect_db.c */
 /* is_slapd_running()
  * returns 1 if slapd is running, 0 if not, -1 on error
